#loading the data file
library(xlsx)
breast <- read.xlsx(file="lda_regression_dataset.xls", header=T, sheetIndex=2)
print(summary(breast))

#absolute frequencies
n1 <- table(breast$target)[1] #begnin
n2 <- table(breast$target)[2] #malignant
n <- n1+n2

#coding the target attribute - Tomassone, page 38
y1 <- n2/n
y2 <- -n1/n
y <- ifelse(breast$target=="begnin",y1,y2)

#linear regression
reg <- lm(y ~ ., data = breast[-4])
print(reg)
beta <- reg$coefficients
print(round(beta,5))

#summary of the regression
sreg <- summary(reg)

#R2 de la rgression
R2 <- sreg$r.squared

#D2 (Mahalanobis distance) - Huberty, page 353 ; Tomassone, page 38
D2 <- (R2/(1-R2))*(n*(n-2))/(n1*n2)
names(D2)[1] <- "D2"
print(D2)

#some values (Tomassone, page 27)
c1 <- n1+n2-2
c2 <- (n1*n2)/(n1+n2)

#rho
rho <- (c1+c2*D2)/(c2*(y1-y2))
names(rho)[1] <- "rho"
print(rho)

#score function before the correction of the intercept
theta <- beta*rho
print(round(theta,5))

#calculations for the correction of the intercept

#prior distribution - 1st correction
e1 <- log(n1/n2)

#global centroid
mu <- sapply(breast[1:3],mean)

#conditional centroids
mu.cond <- aggregate(breast[1:3],by=list(breast$target),mean)[2:4]

#difference
mu.centre <- ((mu.cond[1,]+mu.cond[2,])-2*mu)

#coef. of the LDA without the intercept
coef.lda.p <- theta[2:4]

#scalar product - 2nd correction
e2 <- -0.5*sum(coef.lda.p*mu.centre)

#delta
delta <- e1 + e2
names(delta)[1] <- "delta"
print(delta)

#correction of the intercept
theta_tilde <- theta
theta_tilde[1] <- theta[1] + delta

#score function of the LDA with the correction of the intercept
print(round(theta_tilde,5))

#comparison of the predictive performances

#function for the calculation of the confusion matrix
confusion.matrix <- function(dataset,coef){
  #prediction - one row
  prediction <- function(data.row){
    score <- sum(data.row[1:3]*coef[2:4])+coef[1]
    return(ifelse(score>=0,"begnin","malignant"))
  }
  #prediction for all rows
  pred <- factor(apply(data.matrix(dataset),1,prediction))
  #confusion matrix
  cm <- table(dataset$target,pred)
  print(cm)
  #error rate
  er <- 1-sum(diag(cm))/sum(cm)
  print(er)
}

#confusion matrix for regression
confusion.matrix(breast,beta)

#confusion matrix for lda (linear discriminant analysis)
confusion.matrix(breast,theta_tilde)
